/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "avatar_stats.h"

#include <sstream>

#include <cctype>

#include "main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


AvatarStats::AvatarStats(const AttribList& attrib)
{
	XMLParser::FindAttrib(attrib,"name",name);
	MainForm->DisplayAvatarName(name);
	std::ostringstream	buffer;
	buffer << "ex " << name << "\n";
	MainForm->Send(buffer.str());
}


void	AvatarStats::Clear()
{
	name = "";
	rank = "";
	stamina[CURRENT] = stamina[MAXIMUM] = 0;
	strength[CURRENT] = strength[MAXIMUM] = 0;
	dexterity[CURRENT] = dexterity[MAXIMUM] = 0;
	intelligence[CURRENT] = intelligence[MAXIMUM] = 0;
	cash = loan = 0;
	hc = ak = merchant_pts = trader_pts = 0;
	MainForm->ClearAvatarStatsDisplay();
}

void	AvatarStats::NewStatSelect(const std::string& stat_name)
{
	const std::string	el_names[] =
		{ "Stamina", "Strength", "Dexterity", "Intelligence", " "	};

	int	stat_number = 99999;
	for(int count = 0;el_names[count] != " ";count++)
	{
		if(el_names[count] == stat_name)
		{
			stat_number = count;
			break;
		}
	}

	switch(stat_number)
	{
		case	0:	MainForm->UpdateStatBox(" Stamina ",stamina[CURRENT],stamina[MAXIMUM]);						break;
		case	1:	MainForm->UpdateStatBox(" Strength ",strength[CURRENT],strength[MAXIMUM]);					break;
		case	2:	MainForm->UpdateStatBox(" Dexterity ",dexterity[CURRENT],dexterity[MAXIMUM]);				break;
		case	3:	MainForm->UpdateStatBox(" Intelligence ",intelligence[CURRENT],intelligence[MAXIMUM]);	break;
		default:	MainForm->UpdateStatBox("",0,100);																			break;
	}
}

void	AvatarStats::Process(const AttribList& attrib)
{
	std::string	temp;
	XMLParser::FindAttrib(attrib,"name",temp);
	if(temp != "")
	{
		name = temp;
		MainForm->DisplayAvatarName(name);
		return;
	}
	XMLParser::FindAttrib(attrib,"rank",temp);
	if(temp != "")
	{
		rank = "-" + temp + "-";
		MainForm->DisplayAvatarRank(rank);
		return;
	}
	ProcessStats(attrib);
}

void	AvatarStats::ProcessAKCredits(const AttribList& attrib)
{
	ak = XMLParser::FindNumAttrib(attrib,"amount",0);
	if(ak > -99999)
		MainForm->DisplayAvatarAKCredits(ak);
	else
		MainForm->ClearAvatarAKCredits();
}

void	AvatarStats::ProcessCash(const AttribList& attrib)
{
	std::string	amount;
	XMLParser::FindAttrib(attrib,"amount",amount);
	cash = std::atol(amount.c_str());
	std::ostringstream buffer;
	if(cash < 1000L)
		buffer << cash << "ig";
	else
	{
		if(cash < 1000000L)
			buffer << cash/1000 << "Kig";
		else
			buffer << cash/1000000 << "Mig";
	}
	MainForm->DisplayAvatarCash(buffer.str());
}

void	AvatarStats::ProcessHaulerCredits(const AttribList& attrib)
{
	std::string	amount;
	XMLParser::FindAttrib(attrib,"amount",amount);
	hc = std::atol(amount.c_str());
	if(hc >= 0)
		MainForm->DisplayAvatarHaulerCredits(hc);
	else
		MainForm->ClearAvatarHaulerCredits();
}

void	AvatarStats::ProcessLoan(const AttribList& attrib)
{
	std::string	amount;
	XMLParser::FindAttrib(attrib,"amount",amount);
	loan = std::atol(amount.c_str());
	if(loan > 0)
	{
		std::ostringstream buffer;
		if(loan < 1000L)
			buffer << loan << "ig";
		else
		{
			if(loan < 1000000L)
				buffer << loan/1000 << "Kig";
			else
				buffer << loan/1000000 << "Mig";
		}
		MainForm->DisplayAvatarLoan(buffer.str());
	}
	else
		MainForm->ClearAvatarLoan();
}

void	AvatarStats::ProcessMerchantCredits(const AttribList& attrib)
{
	merchant_pts = XMLParser::FindNumAttrib(attrib,"amount",0);
	if(merchant_pts > -99999)
		MainForm->DisplayAvatarMerchantCredits(merchant_pts);
	else
		MainForm->ClearAvatarMerchantCredits();
}

void	AvatarStats::ProcessStats(const AttribList& attrib)
{
	static const std::string	el_names[] =
	{
		"sta", "str", "dex", "int", "cash", "loan", "hc", "ak",		//  0- 7
		"merchant", "trader", "score",                              //  8-10
		 " "
	};

	int maximum = XMLParser::FindNumAttrib(attrib,"max",0);
	int current = XMLParser::FindNumAttrib(attrib,"cur",0);
	std::string	stat;
	XMLParser::FindAttrib(attrib,"stat",stat);

	int stat_num = 99999;
	for(int count = 0;el_names[count] != " ";count++)
	{
		if(el_names[count] == stat)
		{
			stat_num = count;
			break;
		}
	}

	switch(stat_num)
	{
		case 0:  stamina[MAXIMUM] = maximum;
					stamina[CURRENT] = current;
					MainForm->DisplayAvatarStamina(maximum,current);
					break;
		case 1:	strength[MAXIMUM] = maximum;
					strength[CURRENT] = current;
					MainForm->DisplayAvatarStrength(maximum,current);
					break;
		case 2:	dexterity[MAXIMUM] = maximum;
					dexterity[CURRENT] = current;
					MainForm->DisplayAvatarDexterity(maximum,current);
					break;
		case 3:	intelligence[MAXIMUM] = maximum;
					intelligence[CURRENT] = current;
					MainForm->DisplayAvatarIntelligence(maximum,current);
					break;
		case 4:	ProcessCash(attrib);					break;
		case 5:	ProcessLoan(attrib); 				break;
		case 6:	ProcessHaulerCredits(attrib); 	break;
		case 7:	ProcessAKCredits(attrib); 			break;
		case 8:	ProcessMerchantCredits(attrib); 	break;
		case 9:	ProcessTraderCredits(attrib); 	break;
		case 10: MainForm->DisplayScoreStats();	break;
	}
}

void	AvatarStats::ProcessTraderCredits(const AttribList& attrib)
{
	trader_pts = XMLParser::FindNumAttrib(attrib,"amount",0);
	if(trader_pts > -99999)
		MainForm->DisplayAvatarTraderCredits(trader_pts);
	else
		MainForm->ClearAvatarTraderCredits();
}


