/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "new_ac_code.h"

#include "account_rec.h"
#include "fed2.h"
#include "init.h"
#include "main.h"
#include "new_account.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------


NewAccount::NewAccount()
{
	status = LOGIN;
	account = new AccountRec;
	account->host = "fed2.ibgames.net";
	account->port = 30003;
	NewAccountForm->ReplyTo(this);
	NewAccountForm->ShowModal();
}

NewAccount::~NewAccount()
{
	//
}


int	NewAccount::Accept(const std::string& line)
{
	if(line.find("play using this char") != std::string::npos)
	{
		MainForm->Send("yes\n");
		status = ALL_SENT;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::AccountID(const std::string& line)
{
	if((line.find("numbers only):") != std::string::npos) ||
					(line.find("name for your character.") != std::string::npos))
	{
		MainForm->Send(account->account_id + "\n");
		status = PASSWORD;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::AcSent(const std::string& line)
{
	static const std::string	error("The account ID you have choosen is already in use\n\
Please try another ID.");

	if(line.find("provide a name") != std::string::npos)
	{
		MainForm->Send(account->avatar_name + "\n");
		status = RACE;
	}
	if(line.find("different name.") != std::string::npos)
	{
		MessageDlg(error.c_str(),mtInformation,TMsgDlgButtons() << mbOK,0);
		status = AC_RETRY;
		NewAccountForm->ShowModal();
	}

	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Confirm(const std::string& line)
{
	if(line.find("confirm password:") != std::string::npos)
	{
		MainForm->Send(account->password + "\n");
		status = EMAIL;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

void	NewAccount::Connect()
{
	static const std::string	gender[] = { "male", "female", "neuter" };

	account->host = NewAccountForm->HostEdit->Text.c_str();
	account->port = std::atoi(NewAccountForm->PortEdit->Text.c_str());
	account->avatar_name = NewAccountForm->NameEdit->Text.c_str();
	account->account_id = NewAccountForm->IDEdit->Text.c_str();
	account->password = NewAccountForm->PwEdit->Text.c_str();
	account->current = true;
	account->save = NewAccountForm->SavePwCheck->Checked;
	account->email = NewAccountForm->EMailEdit->Text.c_str();
	account->race = NewAccountForm->RaceEdit->Text.c_str();
	account->gender = gender[NewAccountForm->GenderRadio->ItemIndex];
	account->str = NewAccountForm->StrEdit->Text.c_str();
	account->sta = NewAccountForm->StaEdit->Text.c_str();
	account->dex = NewAccountForm->DexEdit->Text.c_str();
	account->intel = NewAccountForm->IntEdit->Text.c_str();

	Fed2::init->AddAccount(account);
	Fed2::init->Write();

	if(status == LOGIN)
		MainForm->Logon(account->host,account->port,account->avatar_name);
	if(status == AC_RETRY)
	{
		MainForm->Send(account->account_id + "\n");
		status = PASSWORD;
	}
	if(status == AVATAR_RETRY)
      status = AC_SENT;

	NewAccountForm->Close();
}

int	NewAccount::EMail(const std::string& line)
{
	if(line.find("processing agency):") != std::string::npos)
	{
		MainForm->Send(account->email + "\n");
		status = AC_SENT;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Gender(const std::string& line)
{
	if(line.find("choose the gender") != std::string::npos)
	{
		MainForm->Send(account->gender + "\n");
		MainForm->Send("no\n");		// 'no' distribution of equal distribution of stats
		status = STATS;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Login(const std::string& line)
{
	if(line.find("ogin:") != std::string::npos)
	{
		MainForm->Send("new\n");
		status = ACCOUNT_ID;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Password(const std::string& line)
{
	if(line.find("15 letters):") != std::string::npos)
	{
		MainForm->Send(account->password + "\n");
		status = CONFIRM;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::ProcessText(std::string& line)
{
	switch(status)
	{
		case LOGIN:			return(Login(line));
		case ACCOUNT_ID:	return(AccountID(line));
		case PASSWORD:		return(Password(line));
		case CONFIRM:		return(Confirm(line));
		case EMAIL:			return(EMail(line));
		case AC_SENT:		return(AcSent(line));
		case RACE:			return(Race(line));
		case GENDER:		return(Gender(line));
		case STATS:			return(Stats(line));
		case ACCEPT:		return(Accept(line));
		case ALL_SENT:		return(Start(line));
		default:				return(MORE_TO_COME);
	}
}

int	NewAccount::Race(const std::string& line)
{
	if(line.find("choose the race") != std::string::npos)
	{
		MainForm->Send(account->race + "\n");
		status = GENDER;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Start(const std::string& line)
{
	static const std::string	error("The avatar name you have choosen is already in use\n\
Please try another name.");

	if(line.find("inking to Federation") != std::string::npos)
	{
		MainForm->OutputText(line);
		return(FINISHED);
	}
	if(line.find("not available.") != std::string::npos)
	{
		status = AVATAR_RETRY;
		MessageDlg(error.c_str(),mtInformation,TMsgDlgButtons() << mbOK,0);
		NewAccountForm->ShowModal();
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}

int	NewAccount::Stats(const std::string& line)
{
	if(line.find("allocate to strength") != std::string::npos)
		MainForm->Send(account->str + "\n");
	if(line.find("allocate to stamina") != std::string::npos)
		MainForm->Send(account->sta + "\n");
	if(line.find("allocate to dexterity") != std::string::npos)
	{
		MainForm->Send(account->dex + "\n");
		status = ACCEPT;
	}
	MainForm->OutputText(line);
	return(MORE_TO_COME);
}


