/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "server_parser.h"

#include <sstream>

#include <mmsystem.h>
#include <shellapi.h>

#include "avatar_stats.h"
#include "exchange.h"
#include "fed2.h"
#include "fedmap.h"
#include "friends_code.h"
#include "location.h"
#include "main.h"
#include "persona.h"
#include "players.h"
#include "ship_stats.h"
#include "tb_code.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

const int			ServerParser::NOT_AN_ELEMENT = -1;
const std::string	ServerParser::elements[] =
{
	"s-default", "s-add-player", "s-remove-player", "s-new-loc", "s-contents",		//  0- 4
	"s-new-map", "s-remove-contents", "s-map-info", "s-add-factory",				  	//  5- 8
	"s-update-infra", "s-update-workers", "s-remove-factory", "s-player-stats",  	//  9-12
	"s-ship-stats", "s-tb", "s-no-op", "s-spynet-start", "s-spynet",              //	13-17
	"s-examine-start", "s-examine", "s-sound", "s-exch-buy", "s-exch-sell",			// 18-22
	"s-exchange",
	""
};

ServerParser::ServerParser(const std::string& f_name) : XMLParser(f_name)
{
	status = CLEAR;
	current_loc = 0;
}


ServerParser::~ServerParser()
{
	//
}


void	ServerParser::AddContents(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	if(name != "")
		MainForm->AddContents(name);
	status = ADD_CONTENTS;
}

void	ServerParser::AddPlayer(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	if(name != "")
	{
		Fed2::players->Add(name);
		int rank= FindNumAttrib(attrib,"rank",0);
		Fed2::friends->Process(rank,name,true);
	}
	status = ADD_PLAYER;
}

void	ServerParser::DeletePlayer(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	if(name != "")
	{
		Fed2::players->Delete(name);
		Fed2::friends->Process(0,name,false);
	}
	status = DEL_PLAYER;
}

void	ServerParser::EndElement(const std::string& elem)
{
	switch(status)
	{
		case	NEW_LOC:			EndNewLoc();													break;
		case	SPYNET:
		case	EXAMINE:       MainForm->SetSpynetOutputToTop();						break;
//		case	ADD_CONTENTS: 							break;
		case	TB:				Fed2::tight_beam->ProcessOutput(tb_name,tb_data);	break;
	}
	status = CLEAR;
}

void	ServerParser::EndNewLoc()
{
//	MainForm->SetLocToTop();
	if(current_loc != 0)
	{
		Fed2::fed_map->AddLoc(current_loc);
		current_loc = 0;
	}
}

void	ServerParser::Exchange(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	Fed2::exchange->Initialise(name);
}

void	ServerParser::ExchangeBuy(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	int price = FindNumAttrib(attrib,"price",0);
	if(price != 0)
		Fed2::exchange->ExchangeBuy(name,price);
}

void	ServerParser::ExchangeSell(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	int price = FindNumAttrib(attrib,"price",0);
	int stock = FindNumAttrib(attrib,"stock",0);
	if((price > 0) && (stock > 0))
		Fed2::exchange->ExchangeSell(name,price,stock);
}

int	ServerParser::FindElement(const std::string& elem)
{
	for(int count = 0;elements[count] != "";count++)
	{
		if(elements[count] == elem)
			return(count);
	}
	return(NOT_AN_ELEMENT);
}

void	ServerParser::LoadSpynetPicture(const std::string& name)
{
	std::string	file_name = ".\\pictures\\" + name + ".bmp";
	if(access(file_name.c_str(),0) != 0)
	{
		file_name = ".\\pictures\\interference.bmp";
		if(access(file_name.c_str(),0) != 0)
			return;
	}
	MainForm->SpynetImage->Picture->LoadFromFile(file_name.c_str());
}

void	ServerParser::NewLocation(const AttribList& attrib)
{
	current_loc = new Location(attrib);
	MainForm->LocListBox->Clear();
	current_loc->Name2LocOutput();
	current_loc->Name2TextOutput();
	status = NEW_LOC;
}

void	ServerParser::NewMap(const AttribList& attrib)
{
	if(Fed2::fed_map == 0)
		Fed2::fed_map = new FedMap;
	Fed2::fed_map->NewMap(attrib);
	status = NEW_MAP;
}

void	ServerParser::PlayerStats(const AttribList& attrib)
{
	if(Fed2::avatar_stats != 0)
		Fed2::avatar_stats->Process(attrib);
	else
		Fed2::avatar_stats = new AvatarStats(attrib);
	status = STATS;
}

void	ServerParser::PlayFX(const AttribList& attrib)
{
	std::string	text;
	FindAttrib(attrib,"name",text);
	if((text != "") && MainForm->SoundItem->Checked)
	{
		std::ostringstream	buffer;
		buffer << MainForm->HomeDir() << "\\sounds\\" << text << ".wav";
		sndPlaySound(buffer.str().c_str(),SND_ASYNC);
	}
}

void	ServerParser::ProcessShipStats(const AttribList& attrib)
{
	if(Fed2::ship_stats != 0)
		Fed2::ship_stats->Process(attrib);
	else
		Fed2::ship_stats = new ShipStats(attrib);
	status = SHIP_STATS;
}

void	ServerParser::RemoveContents(const AttribList& attrib)
{
	std::string	name;
	FindAttrib(attrib,"name",name);
	if(name != "")
		MainForm->RemoveContents(name);
	status = REMOVE_CONTENTS;
}

void	ServerParser::StartDisplay(const std::string& caption,const std::string& name)
{
	MainForm->TopNotebook->ActivePage = MainForm->ExSpTab;
	MainForm->OutputStory(caption.c_str());
	MainForm->SpynetNamePanel->Caption = caption.c_str();
	MainForm->SpynetOutput->Clear();
//	MainForm->SpynetOutput->Lines->Add(" ");
	LoadSpynetPicture(name);
}

void	ServerParser::StartElement(const std::string& elem,const AttribList& attrib)
{
	switch(FindElement(elem))
	{
		case   0:	status = DEFAULT;							 										break;
		case	 1:	AddPlayer(attrib);						 										break;
		case	 2:	DeletePlayer(attrib);					 										break;
		case   3:	NewLocation(attrib);						 										break;
		case   4:	AddContents(attrib);						 										break;
		case   5:	NewMap(attrib);							 										break;
		case   6:	RemoveContents(attrib);					 										break;
		case	 7:	Fed2::fed_map->MapInfo(attrib);		 	status = MAP_INFO;			break;
		case	 8:	Fed2::fed_map->AddFactory(attrib);	 	status = ADD_FACTORY;		break;
		case	 9:	Fed2::fed_map->DisplayInfra();		 	status = INFRA_UPDATE;		break;
		case	10:	Fed2::fed_map->UpdateWorkers(attrib);	status = UPDATE_WORKERS;	break;
		case	11:	Fed2::fed_map->RemoveFactory(attrib);	status = REMOVE_FACTORY;	break;
		case	12:	PlayerStats(attrib);						 										break;
		case	13:	ProcessShipStats(attrib);                            	            break;
		case	14:	StartTightBeam(attrib);															break;
		case	15:	MainForm->Send("<c-no-op/>\n");			status = NO_OP;				break;
		case  16:	StartSpynet(attrib);																break;
		case	17:	status = SPYNET;																	break;
		case	18:	StartExamine(attrib);															break;
		case	19:	status = EXAMINE;																	break;
		case	20:	PlayFX(attrib);																	break;
		case	21:	ExchangeBuy(attrib);																break;
		case	22:	ExchangeSell(attrib);															break;
		case	23:	Exchange(attrib);																	break;
	}
}

void	ServerParser::StartExamine(const AttribList& attrib)
{
	std::string	sp_name, sp_rank;
	FindAttrib(attrib,"name",sp_name);
	FindAttrib(attrib,"rank",sp_rank);
	std::string	caption = "Examine " + sp_rank + " " + sp_name;
	StartDisplay(caption,sp_name);
	status = EX_START;
}

void	ServerParser::StartSpynet(const AttribList& attrib)
{
	std::string	sp_name, sp_rank;
	FindAttrib(attrib,"name",sp_name);
	FindAttrib(attrib,"rank",sp_rank);
	std::string	caption = "Spynet Report for " + sp_rank + " " + sp_name;
	StartDisplay(caption,sp_name);
	status = SPYNET_START;
}

void	ServerParser::StartTightBeam(const AttribList& attrib)
{
	FindAttrib(attrib,"name",tb_name);
	tb_data = "";
	status = TB;
}

void	ServerParser::TextData(const std::string& text)
{
	switch(status)
	{
		case 	DEFAULT:          if(text.find("+++") != 0)
											MainForm->OutputText(text);
										else
											MainForm->OutputStory(text.c_str());
										break;
		case	NEW_LOC:
		case	ADD_CONTENTS:		
		case	REMOVE_CONTENTS:	MainForm->OutputText(text); 							break;
		case	TB:					tb_data += text;											break;
		case	SPYNET:
		case	EXAMINE:          MainForm->SpynetOutput->Lines->Add(text.c_str());
										MainForm->OutputStory(text.c_str());
										break;
	}
}






