/*-----------------------------------------------------------------------
				FedTerm version 1.60 - Windows Federation 2 Client
			  Copyright (c) 1995-2006 Interactive Broadcasting Ltd

	This program is free software: you can redistribute it and /or modify 
	it under the terms of the GNU General Public License as published by 
	the Free Software Foundation: either version 2 of the License, or (at 
	your option) any later version.

	This program is distributed in the hope that it will be useful, but
	WITHOUT ANY WARRANTY: without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
	General Public License for more details.

	You should have received a copy of the GNU General Public License 
	along with this program; if not, write to the Free Software Foundation, 
	Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307 USA

	Program written and compiled with Borland C++ Builder 5
	Programming and design: 	Alan Lenton
	Home website:					www.ibgames.net
-----------------------------------------------------------------------*/

#include <vcl.h>
#pragma hdrstop

#include "xml.h"

#include <sstream>
#include <cstdlib>

#include "main.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------

XMLParser::XMLParser(const std::string& f_name)
{
	file_name = f_name;
	if(file_name != "")
		file.open(file_name.c_str(),std::ios::in);
}

XMLParser::~XMLParser()
{
	if(file)
		file.close();
}


void	XMLParser::BuildAttributes(const std::string& line)
{
	std::string::size_type	start = 0;
	std::string::size_type	end;
	attrib_list.clear();
	unsigned int len = line.length();
	if(len > 4)
		len -= 4;
	else
		return;

	AttribRec	rec;
	for(;;)
	{
		if(start >= len)
			return;
		end = line.find('=',start);
		if(end == std::string::npos)
			return;
		rec.name = line.substr(start,end - start);
		start = end + 2;
		end = line.find('\'',start);
		if(end == std::string::npos)
			return;
		rec.value = line.substr(start,end - start);
		start = end + 2;
		UnEscapeXML(rec.name);
		UnEscapeXML(rec.value);
		attrib_list.push_back(rec);
	}
}

void	XMLParser::BuildElement(const std::string& line)
{
	std::string::size_type	end = line.find(' ');
	if(end == std::string::npos) 	// no attributes
	{
		element = line;
		attrib_list.clear();
		return;
	}
	element = line.substr(0,end);
	BuildAttributes(line.substr(end + 1));
}

std::string&	XMLParser::EscapeXML(const std::string&text)
{
	static std::string	xml_text;

	std::ostringstream	buffer;
	int len = text.length();
	for(int count = 0;count < len;count++)
	{
		switch(text[count])
		{
			case  '<':	buffer << "&lt;";			break;
			case  '>':	buffer << "&gt;";			break;
			case  '&':	buffer << "&amp;";		break;
			case '\'':	buffer << "&apos;";		break;
			default	:	buffer << text[count];	break;
		}
	}
	xml_text = buffer.str();
	return(xml_text);
}

void	XMLParser::FindAttrib(const AttribList& attrib,const std::string& name,std::string& variable)
{
	for(AttribList::const_iterator iter = attrib.begin();iter != attrib.end();iter++)
	{
		if((*iter).name == name)
		{
			variable = (*iter).value;
			return;
		}
	}
	variable = "";
}

bool	XMLParser::FindBoolAttrib(const AttribList& attrib,const std::string& name,bool default_val)
{
	std::string	temp;
	FindAttrib(attrib,name,temp);
	if(temp == "")
		return(default_val);
	else
		return(temp == "true");
}

int	XMLParser::FindNumAttrib(const AttribList& attrib,const std::string& name,int default_val)
{
	std::string	temp;
	FindAttrib(attrib,name,temp);
	if(temp == "")
		return(default_val);
	else
		return(std::atoi(temp.c_str()));
}

bool	XMLParser::Parse()
{
	if(!file)
		return(false);

	std::string	line;
	int next_posn = 0;
	while(std::getline(file,line))
	{
		if(line.substr(0,5).find("<?xml") != std::string::npos)		// skip xml header line
			continue;
		ProcessLine(line);
	}
	return(true);
}

void	XMLParser::Parse(const std::string& line)
{
	if(line.substr(0,5).find("<?xml") == std::string::npos)			// skip xml header line
		ProcessLine(line);
}

void	XMLParser::ProcessLine(const std::string& line)
{
	std::string::size_type	blanks = line .find_first_not_of(" \t");
	if(blanks == std::string::npos)
		return;
	std::string::size_type	start;
	if(line[blanks] == '<')
		start = line.find('<');
	else
		start = std::string::npos;
	std::string::size_type	end = line.find('>');
	if((start != std::string::npos) && (end != std::string::npos))
	{
		std::string	element_str(line.substr(start + 1,end - start - 1));
		if(element_str[0] == '/')		// is it an end element
		{
			EndElement(element_str.substr(1));
			return;
		}
		// is it an element with just attributes
		if(element_str[element_str.length() - 1] == '/')
		{
			BuildElement(element_str.substr(0,element_str.length() - 1));
			StartElement(element,attrib_list);
			EndElement(element);
			return;
		}
		// must be an element with more than just attributes
		BuildElement(element_str.substr(0,element_str.length()));
		StartElement(element,attrib_list);

		if((end + 1) >= line.length())
			return;
		std::string	line2(line.substr(end + 1));
		std::string::size_type	end2 = line2.find("</");
		if(end2 == std::string::npos)
		{
			UnEscapeXML(line2);
			TextData(line2);
			return;
		}
		else
		{
			std::string	line4(line2.substr(0,end2));
			UnEscapeXML(line4);
			TextData(line4);
			std::string	line3(line2.substr(end2 + 2));
			std::string::size_type	end3 = line3.find('>');
			if(end2 != std::string::npos)
			{
				EndElement(line3.substr(0,end3));
			}
		}
	}
	else
	{
		if(end == std::string::npos)
		{
			std::string	the_line(line);
			UnEscapeXML(the_line);
			TextData(the_line);
		}
		else
		{
			std::string::size_type	start4 = line.find('<');
			std::string::size_type	end4 = line.find('>');
			if((start4 != std::string::npos) && (end4 != std::string::npos))
			{

				std::string line5(line.substr(0,start4));
				UnEscapeXML(line5);
				TextData(line5);
				EndElement(line.substr(start4,end4 - start4));
			}
			else
			{
				std::string	the_line(line);
				UnEscapeXML(the_line);
				TextData(the_line);
			}
		}
	}
}

std::string&	XMLParser::UnEscapeXML(std::string& text)
{
	std::ostringstream	buffer;
	int	len = text.length();
	for(int count = 0;count < len;count++)
	{
		if(text[count] == '&')
		{
			switch(text[count + 1])
			{
				case 'a':
					if(text[count + 2] == 'm')	{ buffer << '&';	count += 4; break; }
					if(text[count + 2] == 'p') { buffer << '\'';	count += 5;}
					break;
				case 'g':	buffer << '>';		count += 3;	break;
				case 'l':	buffer << '<';		count += 3;	break;
				case 'q':	buffer << '\"';   count += 5;	break;
			}
		}
		else
			buffer << text[count];
	}
	text = buffer.str();
	return(text);
}


